@@ -1,6 +1,6 @@
 package Test::Trap::Builder::PerlIO;
 
-use version; $VERSION = qv('0.0.3');
+use version; $VERSION = qv('0.0.4');
 
 use strict;
 use warnings;
@@ -9,12 +9,16 @@ use PerlIO 'scalar';
 
 sub import {
   my $builder = Test::Trap::Builder->new;
-
   $builder->output_layer_backend( perlio => $_ ) for sub {
-    my ($result, $name, $globref) = @_;
-    no warnings 'io';
-    open *$globref, '>', \$result->{$name};
+    my $self = shift;
+    my ($name, $fileno, $globref) = @_;
+    local *$globref;
+    {
+      no warnings 'io';
+      open *$globref, '>', \$self->{$name};
+    }
     $globref->autoflush(1);
+    $self->Next;
   };
 }
 
@@ -28,7 +32,7 @@ Test::Trap::Builder::PerlIO - Output layer backend using PerlIO::scalar
 
 =head1 VERSION
 
-Version 0.0.3
+Version 0.0.4
 
 =head1 DESCRIPTION
 
